#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <dlfcn.h>
#include <imago2.h>

int parse_opt(int argc, char **argv);

char *inpat, *outpat = "out%4d.png";
int range_min, range_max = INT_MAX;
float fps = 30;
char *plugin_name;

void *dl;
int (*init)(void);
void (*shutdown)(void);
int (*process)(struct img_pixmap*, int, float);

int main(int argc, char **argv)
{
	char *fname;
	int i, frame_width = -1, frame_height;

	if(parse_opt(argc, argv) == -1) {
		return 1;
	}

	if(!(dl = dlopen(plugin_name, RTLD_LAZY))) {
		fprintf(stderr, "failed to open post plugin: %s: %s\n", plugin_name, dlerror());
		return 1;
	}
	init = dlsym(dl, "init");
	shutdown = dlsym(dl, "shutdown");
	if(!(process = dlsym(dl, "process"))) {
		fprintf(stderr, "invalid post plugin, failed to load process func: %s\n", dlerror());
		return 1;
	}

	if(init) {
		if(init() == -1) {
			fprintf(stderr, "init failed\n");
			return 1;
		}
	}

	fname = alloca(strlen(inpat) + 11);

	for(i=range_min; i<range_max; i++) {
		struct img_pixmap frame;
		float tmsec = (float)i / fps;

		sprintf(fname, inpat, i);

		img_init(&frame);
		if(img_load(&frame, fname) == -1) {
			fprintf(stderr, "file not found: %s\n", fname);
			img_destroy(&frame);
			break;
		}
		if(frame_width == -1) {
			frame_width = frame.width;
			frame_height = frame.height;
		}
		if(frame_width != frame.width || frame_height != frame.height) {
			fprintf(stderr, "frame %d differs in dimensions! (%dx%d instead of %dx%d)\n", i, frame.width, frame.height, frame_width, frame_height);
			img_destroy(&frame);
			break;
		}

		img_convert(&frame, IMG_FMT_RGB24);
		if(process(&frame, i, tmsec) == -1) {
			fprintf(stderr, "the post plugin signalled an error\n");
			img_destroy(&frame);
			break;
		}

		sprintf(fname, outpat, i);
		if(img_save(&frame, fname) == -1) {
			fprintf(stderr, "failed to save frame %d, aborting\n", i);
			img_destroy(&frame);
			break;
		}
		img_destroy(&frame);
	}

	if(shutdown) {
		shutdown();
	}
	return 0;
}

int parse_opt(int argc, char **argv)
{
	int i;
	char *endp;

	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-') {
			if(argv[i][2] == 0) {
				switch(argv[i][1]) {
				case 'i':
					inpat = argv[++i];
					break;

				case 'o':
					outpat = argv[++i];
					break;

				case 'r':
					fps = strtod(argv[++i], &endp);
					if(endp == argv[i]) {
						fprintf(stderr, "-r must be followed by a number\n");
						return -1;
					}
					break;

				case 's':
					range_min = strtol(argv[++i], &endp, 10);
					if(0) {
				case 'e':
						range_max = strtol(argv[++i], &endp, 10);
					}
					if(endp == argv[i]) {
						fprintf(stderr, "-%c must be followed by a number\n", argv[i - 1][1]);
						return -1;
					}
					break;

				case 'p':
					plugin_name = argv[++i];
					break;

				default:
					goto invalid;
				}
			} else {
				goto invalid;
			}
		} else {
			goto invalid;
		}
	}

	if(!inpat) {
		fprintf(stderr, "you must specify an input image sequence\n");
		return -1;
	}
	if(!plugin_name) {
		fprintf(stderr, "you must specify a post plugin\n");
		return -1;
	}

	printf("%s -> [%s] -> %s\n", inpat, plugin_name, outpat);
	/*if(range_min > 0 || range_max < INT_MAX) {*/
		printf("frames: [%d, ", range_min);
		if(range_max < INT_MAX) {
			printf("%d]\n", range_max);
		} else {
			printf("inf]\n");
		}
	/*}*/
	printf("framerate: %f fps\n", fps);

	return 0;

invalid:
	fprintf(stderr, "invalid argument: %s\n", argv[i]);
	return -1;
}
